// LWA_MCS_subsystems.h: S.W. Ellingson, Virginia Tech, 2009 Aug 02
// ---
// Header file defining things used for LWA MCS software -- subsystem-specific 
// See end of this file for history.

#ifndef LWA_MCS_SUBSYSTEMS_H  /* this keeps this header from getting rolled in more than once */
#define LWA_MCS_SUBSYSTEMS_H

/* === LWA Subsystem IDs === */
#define LWA_MAX_SID 11 /* maximum subsystem code; also maximum number of subsystems */
#define LWA_SID_NU1  1 /* null subsystem #1 (used for testing) */
#define LWA_SID_NU2  2 /* null subsystem #2 (used for testing) */
#define LWA_SID_NU3  3 /* null subsystem #3 (used for testing) */
#define LWA_SID_NU4  4 /* null subsystem #4 (used for testing) */
#define LWA_SID_NU5  5 /* null subsystem #5 (used for testing) */
#define LWA_SID_NU6  6 /* null subsystem #6 (used for testing) */
#define LWA_SID_NU7  7 /* null subsystem #7 (used for testing) */
#define LWA_SID_NU8  8 /* null subsystem #8 (used for testing) */
#define LWA_SID_NU9  9 /* null subsystem #9 (used for testing) */
#define LWA_SID_MCS 10 /* MCS */
#define LWA_SID_SHL 11 /* shelter */

int LWA_getsid( char *ssc ) {
  /* ssc is the three-character subsystem code */
  /* returns the LWA subsystem ID, or 0 if there is an error */
  int sid = 0;
  if (!strcmp(ssc,"NU1")) sid = LWA_SID_NU1;
  if (!strcmp(ssc,"NU2")) sid = LWA_SID_NU2;
  if (!strcmp(ssc,"NU3")) sid = LWA_SID_NU3;
  if (!strcmp(ssc,"NU4")) sid = LWA_SID_NU4;
  if (!strcmp(ssc,"NU5")) sid = LWA_SID_NU5;
  if (!strcmp(ssc,"NU6")) sid = LWA_SID_NU6;
  if (!strcmp(ssc,"NU7")) sid = LWA_SID_NU7;
  if (!strcmp(ssc,"NU8")) sid = LWA_SID_NU8;
  if (!strcmp(ssc,"NU9")) sid = LWA_SID_NU9;
  if (!strcmp(ssc,"MCS")) sid = LWA_SID_MCS;
  if (!strcmp(ssc,"SHL")) sid = LWA_SID_SHL;
  return sid;
  } /* LWA_getsid() */

char *LWA_sid2str( int sid ) {
  /* sid is the LWA subsystem ID */
  /* returns the associated three-character subsystem code */
  /* returns "XXX" if there is an error */
  if (sid == LWA_SID_NU1) return "NU1";
  if (sid == LWA_SID_NU2) return "NU2";
  if (sid == LWA_SID_NU3) return "NU3";
  if (sid == LWA_SID_NU4) return "NU4";
  if (sid == LWA_SID_NU5) return "NU5";
  if (sid == LWA_SID_NU6) return "NU6";
  if (sid == LWA_SID_NU7) return "NU7";
  if (sid == LWA_SID_NU8) return "NU8";
  if (sid == LWA_SID_NU9) return "NU9";
  if (sid == LWA_SID_MCS) return "MCS";
  if (sid == LWA_SID_SHL) return "SHL";
  return "XXX";
  } /* LWA_sid2str() */

/* === LWA Command (TYPE) IDs === */
#define LWA_MAX_CMD  3 /* maximum code; also maximum number of commands */
#define LWA_CMD_MCSSHT 0 /* Not a subsystem command.  Directs ms_mcic to shutdown */
#define LWA_CMD_PNG    1 /* PNG */
#define LWA_CMD_RPT    2 /* RPT */
#define LWA_CMD_SHT    3 /* SHT */

int LWA_getcmd( char *ssc ) {
  /* ssc is the three-character command (TYP) */
  /* returns the LWA command ("TYPE"), or 0 if there is an error */
  int cmd = 0;
  if (!strcmp(ssc,"PNG")) cmd = LWA_CMD_PNG;
  if (!strcmp(ssc,"RPT")) cmd = LWA_CMD_RPT;
  if (!strcmp(ssc,"SHT")) cmd = LWA_CMD_SHT;
  return cmd;
  } /* LWA_getcmd() */

char *LWA_cmd2str( int cmd ) {
  /* ssc is the three-character command (TYP) */
  /* returns the LWA command ("TYPE"), or "   " if there is an error */
  if (cmd == LWA_CMD_MCSSHT) return "SHT";
  if (cmd == LWA_CMD_PNG)    return "PNG";
  if (cmd == LWA_CMD_RPT)    return "RPT";
  if (cmd == LWA_CMD_SHT)    return "SHT";
  return "   ";
  } /* LWA_getsid() */

///* types of messages that can be sent to ms_mcic subsystem handlers */
//#define LWA_MTYPE_PING 1 /* ping; just checking */

#endif /* #ifndef LWA_MCS_SUBSYSTEMS_H */

//==================================================================================
//=== HISTORY ======================================================================
//==================================================================================
// Aug 02, 2009: Added LWA_CMD_MCSSHT (svn rev 21)
// Aug 01, 2009: Added UT Day/Time to log files (svn rev 19)
//               (svn rev 20)
// Jul 31, 2009: Pulled out from LWA_MCS.h  (svn rev 18)
