// msei.c: S.W. Ellingson, Virginia Tech, 2009 Aug 02
// ---
// COMPILE: gcc -o msei msei.c
// ---
// COMMAND LINE: msei <dest> <cmd> <data>
//   <dest> = DESTINATION.  (three-letter subsystem designator)
//            see LWA_MCS.h for valid entries
//   <cmd>  = TYPE. (three-letter command TYPE)
//            see LWA_MCS.h for valid entries
//   <data> = DATA. (DATA field for given TYPE) -- OPTIONAL (depends on command)
// ---
// REQUIRES: 
// ---
// Used to inject commands into MCS/Scheduler's ms_exec process.  
//   (This is the client-side of a client-server connection)
// See end of this file for history.

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h> /* for network sockets */
#include <arpa/inet.h>  /* for network sockets */

#include "LWA_MCS.h"

#define MY_NAME "msei (v.20090802.1)"
#define ME "7" 

#define B 256

main ( int narg, char *argv[] ) {

  /*=================*/
  /*=== Variables ===*/
  /*=================*/

  int sockfd;                 /* socket file discriptor */
  struct sockaddr_in address; /* for network sockets */

  //int len;
  int result;

  char dest[3];
  char cmd[3];
  char data[B];
  //int sid;
  //int cid;

  struct LWA_cmd_struct c; 

  struct timeval tv;  /* from sys/time.h; included via LWA_MCS.h */
  struct timezone tz;

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  //printf("I am %s [%s]\n",MY_NAME,ME);

  if (narg>1) { 
    sscanf(argv[1],"%3s",dest);
    //printf("dest = <%s>\n",dest);
    } else {
    printf("[%s] FATAL: dest not provided\n",ME);
    return;
    } 
  if (narg>2) { 
    sscanf(argv[2],"%3s",cmd);
    //printf("dest = <%s>\n",dest);
    } else {
    printf("[%s] FATAL: cmd not provided\n",ME);
    return;
    } 
  strcpy(data,"");
  if (narg>3) { 
    sscanf(argv[3],"%s",data);
    //printf("data = <%s>\n",data);
    } 


  /*=== Fill in structure to send to ms_exec... ===*/

  /* figure out what subsystem */
  if ( !( c.sid = LWA_getsid( dest ) ) ) {
    printf("[%s] FATAL: subsystem <%s> not recognized\n",ME,dest);
    return;    
    }

  /* Outbound value of ref doesn't matter. (Gets assigned by ms_exec.) */
  c.ref = 0;     

  /* figure out what command */
  if ( !( c.cid = LWA_getcmd( cmd ) ) ) {
    printf("[%s] FATAL: cmd <%s> not recognized\n",ME,cmd);
    return;    
    }  
  //printf("[%s] c.cid=%d\n",ME,c.cid);

  /* not implemented yet */
  c.subslot = 0; 

  /* Not to be scheduled; just do as soon as time permits */
  c.bScheduled = 0;

  /* Outbound value sent to current time; won't matter because it's not */
  /* scheduled (see above) */
  gettimeofday( &c.tv, &tz );

  /* Outbound value doesn't matter, but gets changed by ms_exec */
  c.bAccept = 0; 

  /* Outbound value doesn't matter, but gets changed by ms_exec */
  c.eSummary = LWA_SIDSUM_NULL; 

  /* Outbound value doesn't matter, but gets changed by ms_mcic */
  c.eMIBerror = LWA_MIBERR_OK;

  /* Gets changed by ms_exec */
  strcpy(c.data,data); /* changed in reply */

  /* create socket */
  sockfd = socket(
                  AF_INET,     /* domain; network sockets */
                  SOCK_STREAM, /* type (TCP-like) */
                  0);          /* protocol (normally 0) */
    
  /* network sockets */
  address.sin_family = AF_INET;
  address.sin_addr.s_addr = inet_addr(LWA_IP_MSE);
  address.sin_port = htons(LWA_PORT_MSE);

  /* connect socket to server's socket */
  result = connect( sockfd, (struct sockaddr *) &address, sizeof(address));
  if (result==-1) {
    perror("msei");
    return;
    }

  write(sockfd, &c, sizeof(struct LWA_cmd_struct));
  read(sockfd,&c,sizeof(struct LWA_cmd_struct));
  //printf("saw %d %d %d\n",c.sid,c.cid,c.ref);
  printf("[%s] ref=%ld, bAccept=%d, eSummary=%d, data=<%s>\n",ME,c.ref,c.bAccept,c.eSummary,c.data);

  close(sockfd); 
  return;

  } /* main() */

//==================================================================================
//=== HISTORY ======================================================================
//==================================================================================
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Aug 02
//   .1: Cleaning up console output
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Jul 26
//   .1: Added LWA_cmd_struct field eMIBerror (svn rev 10)
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Jul 23
//   .1: Adding "DATA" command line argument (svn rev 7)
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Jul 20
//   .1: Adding "time to execute" field to command struct
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Jul 17
//   .1: Cleaning up; esp. implementation of LWA_cmd_struct
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Jul 13
//   .1: 
// msei.c: S.W. Ellingson, Virginia Tech, 2009 Jul 10
//   .1: preliminary version (not even numbered); 3-letter char as argument 
//   .2: changing command line and transfer format

//==================================================================================
//=== BELOW THIS LINE IS SCRATCH ===================================================
//==================================================================================

